#include <bits/stdc++.h>

using namespace std;

const int maxn = 200005;
const int maxm = 400005;

int n, m, t;

struct Edge
{
    int x;
    int y;
    int w;
};

Edge edges[maxm];

struct DSU
{
    int par[maxn];
    int sz[maxn];

    void init()
    {
        for (int i = 1; i <= n; ++i)
        {
            par[i] = i;
            sz[i] = 1;
        }
    }

    int root(int node)
    {
        if (node == par[node])
        {
            return node;
        }

        return par[node] = root(par[node]);
    }

    void connect(int x, int y)
    {
        x = root(x);
        y = root(y);

        if (x == y)
        {
            return;
        }

        if (sz[x] < sz[y])
        {
            par[x] = y;
            sz[y] += sz[x];
        }
        else
        {
            par[y] = x;
            sz[x] += sz[y];
        }
    }

    bool areConnected(int x, int y)
    {
        return root(x) == root(y);
    }
};

DSU dsu;

pair<int, int> queries[maxn];
int answers[maxn];

vector<int> tree[4 * maxn];

struct ParallelNode
{
    int node;
    int level;
    int l;
    int r;

    ParallelNode()
    {

    }

    ParallelNode(int _node, int _level, int _l, int _r)
    {
        node = _node;
        level = _level;
        l = _l;
        r = _r;
    }
};

void fastIO()
{
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
}

int main()
{
    fastIO();

    cin >> n >> m >> t;

    for (int i = 1; i <= m; ++i)
    {
        cin >> edges[i].x >> edges[i].y >> edges[i].w;
    }

    sort(edges + 1, edges + m + 1, [&](const Edge& A, const Edge& B)
    {
        return A.w < B.w;
    });

    for (int i = 1; i <= t; ++i)
    {
        int x, k;
        cin >> x >> k;

        queries[i] = {x, k};
    }

    for (int i = 1; i <= t; ++i)
    {
        tree[1].push_back(i);
    }

    for (int i = 1; i <= t; ++i)
    {
        answers[i] = edges[m].w;
    }

    queue<ParallelNode> q;
    q.push(ParallelNode(1, 0, 1, m));

    int prvLevel = -1;

    while (!q.empty())
    {
        ParallelNode curr = q.front();

        q.pop();

        if (curr.level != prvLevel)
        {
            prvLevel = curr.level;
            dsu.init();
        }

        int mid = (curr.l + curr.r) / 2;

        int queryNode, queryK;
        int node = curr.node;

        for (int i = 1; i <= mid; ++i)
        {
            dsu.connect(edges[i].x, edges[i].y);
        }

        if (curr.l == curr.r)
        {
            for (int i = 0; i < tree[curr.node].size(); ++i)
            {
                if (dsu.sz[dsu.root(queryNode)] > queryK) answers[tree[curr.node][i]] = min(answers[tree[curr.node][i]], edges[mid].w);
            }

            continue;
        }

        //cout << node << " " << curr.l << " " << curr.r << " :: " << curr.level << " -> " << edges[mid].w << endl;

        /*for (int i = 0; i < tree[node].size(); ++i)
        {
            cout << "Query " << tree[node][i] << endl;
        }*/

        for (int i = 0; i < tree[node].size(); ++i)
        {
            queryNode = queries[tree[node][i]].first;
            queryK = queries[tree[node][i]].second;

            if (dsu.sz[dsu.root(queryNode)] > queryK)
            {
                answers[tree[node][i]] = min(answers[tree[node][i]], edges[mid].w);

                //cout << "now " << tree[node][i] << " :: " << answers[tree[node][i]] << endl;

                if (curr.l <= mid)
                {
                    tree[2 * node].push_back(tree[node][i]);
                }
            }
            else
            {
                if (mid + 1 <= curr.r)
                {
                    tree[2 * node + 1].push_back(tree[node][i]);
                }
            }
        }

        if (curr.l <= mid && !tree[2 * node].empty())
        {
            q.push(ParallelNode(2 * node, curr.level + 1, curr.l, mid));
        }

        if (mid + 1 <= curr.r && !tree[2 * node + 1].empty())
        {
            q.push(ParallelNode(2 * node + 1, curr.level + 1, mid + 1, curr.r));
        }
    }

    for (int i = 1; i <= t; ++i)
    {
        cout << answers[i] << "\n";
    }

    return 0;
}
